@@ -1,6 +1,6 @@
 import six
 import re
-from warnings import warn
+import warnings
 from abc import abstractproperty
 from difflib import get_close_matches
 from itertools import starmap, chain, repeat
@@ -14,15 +14,19 @@ from psyplot import rcParams
 from psyplot.compat.pycompat import map
 from psyplot.docstring import docstrings
 from psyplot.data import InteractiveList, _infer_interval_breaks
-from psyplot.plotter import Formatoption, START, DictFormatoption, END
-from psy_simple.plotters import (
-    Base2D, Plot2D, BasePlotter, BaseVectorPlotter, VectorPlot, CombinedBase,
-    DataTicksCalculator, round_to_05, Density, ContourLevels,
-    Simple2DBase, DataGrid, VectorColor, get_cmap)
+from psyplot.plotter import (
+    Formatoption, START, DictFormatoption, END, BEFOREPLOTTING)
+import psy_simple.plotters as psyps
 from psy_maps.boxes import lonlatboxes
 from psy_simple.colors import FixedBoundaryNorm
 
 
+wrap_proj_types = (ccrs._RectangularProjection,
+                   ccrs._WarpedRectangularProjection,
+                   ccrs.InterruptedGoodeHomolosine,
+                   ccrs.Mercator)
+
+
 @docstrings.get_sectionsf('shiftdata')
 def shiftdata(lonsin, datain, lon_0):
     """
@@ -147,6 +151,8 @@ class ProjectionBase(Formatoption):
         northpole   :class:`cartopy.crs.NorthPolarStereo`
         southpole   :class:`cartopy.crs.SouthPolarStereo`
         ortho       :class:`cartopy.crs.Orthographic`
+        stereo      :class:`cartopy.crs.Stereographic`
+        near        :class:`cartopy.crs.NearsidePerspective`
         =========== ======================================="""
 
     projections = {
@@ -157,11 +163,15 @@ class ProjectionBase(Formatoption):
         'northpole': ccrs.NorthPolarStereo,
         'southpole': ccrs.SouthPolarStereo,
         'ortho': ccrs.Orthographic,
+        'stereo': ccrs.Stereographic,
+        'near': ccrs.NearsidePerspective,
         }
 
     projection_kwargs = dict(
         chain(zip(projections.keys(), repeat(['central_longitude']))))
     projection_kwargs['ortho'] = ['central_longitude', 'central_latitude']
+    projection_kwargs['stereo'] = ['central_longitude', 'central_latitude']
+    projection_kwargs['near'] = ['central_longitude', 'central_latitude']
 
     def set_projection(self, value, *args, **kwargs):
         if isinstance(value, ccrs.CRS):
@@ -241,7 +251,8 @@ class BoxBase(Formatoption):
         (contains longitude-latitude boxes of different countries and
         continents)
     [lonmin, lonmax, latmin, latmax]
-        The surrounding longitude-latitude that shall be used
+        The surrounding longitude-latitude that shall be used. Values can be
+        either a float or a string as above
 
     See Also
     --------
@@ -280,7 +291,7 @@ class BoxBase(Formatoption):
             if similar_keys:
                 message += " Maybe you mean on of " + ', '.join(
                     similar_keys)
-            warn(message, RuntimeWarning)
+            warnings.warn(message, RuntimeWarning)
             return
         return [boxes[:, 0].min(), boxes[:, 1].max(),
                 boxes[:, 2].min(), boxes[:, 3].max()]
@@ -398,6 +409,18 @@ class LonLatBox(BoxBase):
 
     dependencies = ['transform']
 
+    @property
+    def lonlatbox_transformed(self):
+        value = np.asarray(self.lonlatbox)
+        transformed = self.transform.projection.transform_points(
+            ccrs.PlateCarree(), value[:2], value[2:])[..., :2]
+        value[:2] = transformed[..., 0]
+        value[2:] = transformed[..., 1]
+        if value[0] == value[1] and isinstance(self.transform.projection,
+                                               ccrs.PlateCarree):
+            value[1] += 360
+        return value
+
     def data_dependent(self, data, set_data=True):
         if isinstance(data, InteractiveList):
             data = data[0]
@@ -455,15 +478,20 @@ class LonLatBox(BoxBase):
                 if value is None:
                     self.lonlatbox = self.data_lonlatbox
                     return data
+            value = list(value)
+            for i, v in enumerate(value):
+                if isinstance(v, six.string_types):
+                    value[i] = self.lola_from_pattern(v)[i]
             is_unstructured = decoder.is_unstructured(
                 base_var if base_var is not None else data)
             is_rectilinear = lon.ndim == 1 and not is_unstructured
             shift = isinstance(self.transform.projection, ccrs.PlateCarree)
             if is_rectilinear and shift:
                 data = data.copy(True)
-                lon_da, data.values = self.shiftdata(
+                lon, data.values = self.shiftdata(
                     lon, data.values, np.mean(value[:2]))
-                data.lon.values = lon_da
+                lon_name = decoder.get_x(data, data.coords).name
+                data[lon_name].values = lon
             elif is_unstructured and shift:
                 # make sure that we are inside the map extent
                 ret = self.transform.projection.transform_points(
@@ -471,15 +499,14 @@ class LonLatBox(BoxBase):
                 lon = ret[..., 0]
                 lat = ret[..., 1]
             self.lonlatbox = value
-            # transform the lonlatbox to the correct projection
-            value = np.array(value)
-            transformed = self.transform.projection.transform_points(
-                ccrs.PlateCarree(), value[:2], value[2:])[..., :2]
-            value[:2] = transformed[..., 0]
-            value[2:] = transformed[..., 1]
-            self.lonlatbox_transformed = value
-            lat_values = value[3:1:-1] if (lat[1:] < lat[:-1]).all() else \
-                value[2:]
+            value = self.lonlatbox_transformed
+            c = warnings.catch_warnings()
+            warnings.filterwarnings('ignore', 'invalid value encountered',
+                                    RuntimeWarning)
+            if is_rectilinear and (lat[1:] < lat[:-1]).all():
+                lat_values = value[3:1:-1]
+            else:
+                lat_values = value[2:]
             if is_rectilinear:
                 kwargs = dict(zip(
                     data.dims[-2:], starmap(slice, [lat_values, value[:2]])))
@@ -529,9 +556,12 @@ class LonLatBox(BoxBase):
 
     def calc_lonlatbox(self, lon, lat):
         if isinstance(self.transform.projection, ccrs.PlateCarree):
-            lon = lon[np.all([lon >= -180, lon <= 360], axis=0)]
-            lat = lat[np.all([lat >= -90, lat <= 90], axis=0)]
-            return [lon.min(), lon.max(), lat.min(), lat.max()]
+            with warnings.catch_warnings():
+                warnings.filterwarnings('ignore', 'invalid value encountered',
+                                        RuntimeWarning)
+                lon = lon[np.all([lon >= -180, lon <= 360], axis=0)]
+                lat = lat[np.all([lat >= -90, lat <= 90], axis=0)]
+                return [lon.min(), lon.max(), lat.min(), lat.max()]
         else:
             if lon.ndim == 1:
                 lon, lat = np.meshgrid(lon, lat)
@@ -603,6 +633,7 @@ class MapExtent(BoxBase):
     update_after_plot = True
 
     def update(self, value):
+
         set_global = False
         if isinstance(value, six.string_types):
             if value == 'global':
@@ -611,18 +642,33 @@ class MapExtent(BoxBase):
                 value = self.lola_from_pattern(value)
         elif value is None:
             value = self.lonlatbox.lonlatbox
+        else:
+            value = list(value)
+            for i, v in enumerate(value):
+                if isinstance(v, six.string_types):
+                    value[i] = self.lola_from_pattern(v)[i]
         # Since the set_extent method does not always work and the data limits
         # are not always correctly set, we test here whether the wished
         # extent (the value) is almost global. If so, we set it to a global
         # value
+        value = list(value)
         with self.ax.hold_limits():
             self.ax.set_global()
             x1, x2, y1, y2 = self.ax.get_extent(ccrs.PlateCarree())
         x_rng = 360. if x1 == x2 else x2 - x1
+        proj = self.ax.projection
         if set_global or ((value[1] - value[0]) / (x_rng) > 0.95 and
                           (value[3] - value[2]) / (y2 - y1) > 0.95):
             self.logger.debug("Setting to global extent...")
             self.ax.set_global()
+            return
+        elif (isinstance(proj,
+                         (ccrs.Orthographic, ccrs.Stereographic)) and
+              np.abs(np.diff(value[:2])) > 350):
+            # HACK: to make sure, that we don't get troubles with the
+            # transformation, we don't go until 360
+            value[:2] = 0, 359.9999
+            self.ax.set_extent(value, crs=ccrs.PlateCarree())
         else:
             try:
                 self.ax.set_extent(value, crs=ccrs.PlateCarree())
@@ -632,6 +678,62 @@ class MapExtent(BoxBase):
                     exc_info=True)
 
 
+class ClipAxes(Formatoption):
+    """
+    Clip the part outside the latitudes of the map extent
+
+    Possible types
+    --------------
+    None
+        Clip if all longitudes are shown (i.e. the extent goes from -180 to
+        180) and the projection is orthographic or stereographic
+    bool
+        True, clip, else, don't
+
+    Notes
+    -----
+    If the plot is clipped. You might need to update with `replot=True`!
+    """
+
+    _orig_path = None
+
+    priority = BEFOREPLOTTING
+
+    connections = ['lonlatbox', 'map_extent']
+
+    def draw_circle(self):
+        import matplotlib.path as mpath
+        self._orig_path = self._orig_path or self.ax.patch.get_path()
+        theta = np.linspace(0, 2*np.pi, 100)
+        center, radius = [0.5, 0.5], 0.5
+        verts = np.vstack([np.sin(theta), np.cos(theta)]).T
+        circle = mpath.Path(verts * radius + center)
+        self.ax.set_boundary(circle, transform=self.ax.transAxes)
+
+    def update(self, value):
+        if value:
+            self.draw_circle()
+        elif value is None:
+            proj = self.ax.projection
+            extent = self.map_extent.value or self.lonlatbox.lonlatbox
+            if (isinstance(proj, (ccrs.Orthographic, ccrs.Stereographic)) and
+                    extent != 'global' and np.abs(np.diff(extent[:2])) > 350):
+                self.draw_circle()
+            else:
+                self.remove()
+        else:
+            self.remove()
+
+    def remove(self):
+        if self._orig_path is not None:
+            try:
+                self.ax.set_boundary(self._orig_path,
+                                     transform=self.ax.transAxes)
+            except Exception:
+                pass
+            del self._orig_path
+
+
 class Transform(ProjectionBase):
     """
     Specify the coordinate system of the data
@@ -669,17 +771,77 @@ class LSM(Formatoption):
         True: draw the continents with a line width of 1
         False: don't draw the continents
     float
-        Specifies the linewidth of the continents"""
+        Specifies the linewidth of the continents
+    str
+        The resolution of the land-sea mask (see the
+        :meth:`cartopy.mpl.geoaxes.GeoAxesSubplot.coastlines` method. Usually
+        one of ``('110m', '50m', '10m')``.
+    list [str or bool, float]
+        The resolution and the linewidth"""
 
     name = 'Land-Sea mask'
 
+    lsm = None
+
     def update(self, value):
-        if value:
-            value = 1.0 if value is True else value
-            self.lsm = self.ax.coastlines(linewidth=value)
-        elif hasattr(self, 'lsm'):
-            self.lsm.remove()
-            del self.lsm
+        self.remove()
+        res, lw = value
+        if res:
+            args = (res, ) if isinstance(res, six.string_types) else ()
+            self.lsm = self.ax.coastlines(*args, linewidth=lw)
+
+    def remove(self):
+        if self.lsm is not None:
+            try:
+                self.lsm.remove()
+            except ValueError:
+                pass
+
+
+class StockImage(Formatoption):
+    """
+    Display a stock image on the map
+
+    This formatoption uses the :meth:`cartopy.mpl.geoaxes.GeoAxes.stock_img`
+    method to display a downsampled version of the Natural Earth shaded relief
+    raster on the map
+
+    Possible types
+    --------------
+    bool
+        If True, the image is displayed
+    """
+
+    image = None
+
+    priority = END
+
+    name = 'Display Natural Earth shaded relief raster'
+
+    connections = ['plot']
+
+    def update(self, value):
+        if value and self.image is None:
+            self.image = self.ax.stock_img()
+            # display below the plot
+            try:
+                mappable = self.plot.mappable
+            except AttributeError:
+                pass
+            else:
+                try:
+                    self.image.zorder = mappable.zorder - 0.1
+                except AttributeError:
+                    self.image.zorder = (
+                        mappable.collections[0].zorder - 0.1)
+        elif not value and self.image is not None:
+            self.remove()
+
+    def remove(self):
+        if self.image is None:
+            return
+        self.image.remove()
+        del self.image
 
 
 class GridColor(Formatoption):
@@ -742,7 +904,7 @@ class GridLabels(Formatoption):
                     self.ax, self.ax.projection, draw_labels=test_value)
             except TypeError as e:  # labels cannot be drawn
                 if value:
-                    warn(e.message, RuntimeWarning)
+                    warnings.warn(e.message, RuntimeWarning)
                 value = False
             else:
                 value = True
@@ -813,7 +975,7 @@ class GridLabelSize(Formatoption):
 
 
 @docstrings.get_sectionsf('GridBase', sections=['Possible types', 'See Also'])
-class GridBase(DataTicksCalculator):
+class GridBase(psyps.DataTicksCalculator):
     """
     Abstract base class for x- and y- grid lines
 
@@ -862,9 +1024,14 @@ class GridBase(DataTicksCalculator):
         else:
             loc = ticker.FixedLocator(value)
         try:
-            zorder = self.plot.mappable.get_zorder()
+            mappable = self.plot.mappable
         except AttributeError:
-            zorder = self.plot.mappable.collections[0].get_zorder()
+            zorder = 0.9
+        else:
+            try:
+                zorder = mappable.zorder
+            except AttributeError:
+                    zorder = mappable.collections[0].zorder
         self._gridliner = self.ax.gridlines(
             crs=ccrs.PlateCarree(), zorder=zorder + 0.1,
             **self.get_kwargs(loc))
@@ -901,7 +1068,7 @@ class GridBase(DataTicksCalculator):
 
     def _round_min_max(self, vmin, vmax):
         exp = np.floor(np.log10(abs(vmax - vmin)))
-        return round_to_05([vmin, vmax], exp, mode='s')
+        return psyps.round_to_05([vmin, vmax], exp, mode='s')
 
 
 class XGrid(GridBase):
@@ -966,11 +1133,36 @@ class YGrid(GridBase):
     axis = 'y'
 
 
-class MapPlot2D(Plot2D):
-    __doc__ = Plot2D.__doc__
+class MapPlot2D(psyps.Plot2D):
+    __doc__ = psyps.Plot2D.__doc__
     # fixes the plot of unstructured triangular data on round projections
 
-    connections = Plot2D.connections + ['transform']
+    connections = psyps.Plot2D.connections + ['transform', 'lonlatbox']
+
+    dependencies = psyps.Plot2D.dependencies + ['clip']
+
+    @property
+    def array(self):
+        ret = super(MapPlot2D, self).array
+        xcoord = self.xcoord
+        if xcoord.ndim == 2 and isinstance(self.transform.projection,
+                                           ccrs.PlateCarree):
+            lon = xcoord.values
+            lat = self.ycoord.values
+            ret[np.any([lon <= -200, lon >= 400, lat <= -120, lat >= 120],
+                       axis=0)] = np.nan
+        return ret
+
+    def _get_xy_pcolormesh(self):
+        """Use the bounds for non-global circumpolar plots"""
+        interp_bounds = self.interp_bounds.value
+        if interp_bounds is None and self.decoder.is_circumpolar(
+                self.raw_data):
+            lola = self.lonlatbox
+            if (isinstance(self.transform.projection, ccrs.PlateCarree) and
+                    np.abs(np.diff(lola.data_lonlatbox[:2])[0]) < 355):
+                return self.xbounds, self.ybounds
+        return super(MapPlot2D, self)._get_xy_pcolormesh()
 
     def _contourf(self):
         t = self.ax.projection
@@ -979,8 +1171,8 @@ class MapPlot2D(Plot2D):
                              ' Spherical contouring is not supported - '
                              ' consider using PlateCarree/RotatedPole.')
         elif self.decoder.is_triangular(self.raw_data):
-            warn('Filled contour plots of triangular data are not correctly '
-                 'warped around!')
+            warnings.warn('Filled contour plots of triangular data are not '
+                          'correctly warped around!')
         return super(MapPlot2D, self)._contourf()
 
     def _tripcolor(self):
@@ -1013,7 +1205,7 @@ class MapPlot2D(Plot2D):
         except AttributeError:
             arr = self.array
             N = len(np.unique(self.bounds.norm(arr.ravel())))
-        cmap = get_cmap(self.cmap.value, N)
+        cmap = psyps.get_cmap(self.cmap.value, N)
         if hasattr(self, '_plot'):
             self.logger.debug("Updating plot")
             self._plot.update(dict(cmap=cmap, norm=self.bounds.norm))
@@ -1061,9 +1253,9 @@ class MapPlot2D(Plot2D):
         return super(MapPlot2D, self).add2format_coord(x, y)
 
 
-class MapDataGrid(DataGrid):
+class MapDataGrid(psyps.DataGrid):
 
-    __doc__ = DataGrid.__doc__ + '\n' + docstrings.dedents("""
+    __doc__ = psyps.DataGrid.__doc__ + '\n' + docstrings.dedents("""
     See Also
     --------
     xgrid
@@ -1101,10 +1293,10 @@ class MapDataGrid(DataGrid):
             triangles.set_mask(mask)
         return triangles
 
-    triangles = property(triangles, doc=DataGrid.triangles.__doc__)
+    triangles = property(triangles, doc=psyps.DataGrid.triangles.__doc__)
 
 
-class MapDensity(Density):
+class MapDensity(psyps.Density):
     """
     Change the density of the arrows
 
@@ -1116,11 +1308,11 @@ class MapDensity(Density):
         if all(val == 1.0 for val in value):
             self.plot._kwargs.pop('regrid_shape', None)
         elif self.decoder.is_unstructured(self.raw_data):
-            warn("Quiver plot of unstructered data does not support the "
-                 "density keyword!", RuntimeWarning)
+            warnings.warn("Quiver plot of unstructered data does not support "
+                          "the density keyword!", RuntimeWarning)
         elif self.decoder.is_circumpolar(self.raw_data):
-            warn("Quiver plot of circumpolar data does not support the "
-                 "density keyword!", RuntimeWarning)
+            warnings.warn("Quiver plot of circumpolar data does not support "
+                          "the density keyword!", RuntimeWarning)
         else:
             shape = self.data.shape[-2:]
             value = map(int, [value[0]*shape[0], value[1]*shape[1]])
@@ -1130,31 +1322,31 @@ class MapDensity(Density):
         self.plot._kwargs.pop('regrid_shape', None)
 
 
-class MapVectorColor(VectorColor):
+class MapVectorColor(psyps.VectorColor):
 
-    __doc__ = VectorColor.__doc__
+    __doc__ = psyps.VectorColor.__doc__
 
     def _maybe_ravel(self, arr):
         # no need to ravel the data for quiver plots
         return np.asarray(arr)
 
 
-class MapVectorPlot(VectorPlot):
+class MapVectorPlot(psyps.VectorPlot):
 
-    __doc__ = VectorPlot.__doc__
+    __doc__ = psyps.VectorPlot.__doc__
 
-    dependencies = VectorPlot.dependencies + ['lonlatbox', 'transform', 'clon',
-                                              'clat']
+    dependencies = psyps.VectorPlot.dependencies + ['lonlatbox', 'transform',
+                                                    'clon', 'clat', 'clip']
 
     def set_value(self, value, *args, **kwargs):
         # stream plots for circumpolar grids is not supported
         if (value == 'stream' and self.raw_data is not None and
                 self.decoder.is_circumpolar(self.raw_data[0])):
-            warn('Cannot make stream plots of circumpolar data!')
+            warnings.warn('Cannot make stream plots of circumpolar data!')
             value = 'quiver'
         super(MapVectorPlot, self).set_value(value, *args, **kwargs)
 
-    set_value.__doc__ = VectorPlot.set_value.__doc__
+    set_value.__doc__ = psyps.VectorPlot.set_value.__doc__
 
     def _get_data(self):
         data = self.data
@@ -1182,7 +1374,7 @@ class MapVectorPlot(VectorPlot):
 
     def _stream_plot(self):
         if self.decoder.is_circumpolar(self.raw_data):
-            warn('Cannot make stream plots of circumpolar data!')
+            warnings.warn('Cannot make stream plots of circumpolar data!')
             return
         # update map extent such that it fits to the data limits (necessary
         # because streamplot scales the density based upon it). This however
@@ -1225,7 +1417,7 @@ class CombinedMapVectorPlot(MapVectorPlot):
         super(CombinedMapVectorPlot, self).update(*args, **kwargs)
 
 
-class MapPlotter(Base2D):
+class MapPlotter(psyps.Base2D):
     """Base plotter for visualizing data on a map
     """
 
@@ -1241,6 +1433,7 @@ class MapPlotter(Base2D):
     clat = CenterLat('clat')
     lonlatbox = LonLatBox('lonlatbox')
     lsm = LSM('lsm')
+    stock_img = StockImage('stock_img')
     grid_color = GridColor('grid_color')
     grid_labels = GridLabels('grid_labels')
     grid_labelsize = GridLabelSize('grid_labelsize')
@@ -1249,6 +1442,7 @@ class MapPlotter(Base2D):
     ygrid = YGrid('ygrid')
     map_extent = MapExtent('map_extent')
     datagrid = MapDataGrid('datagrid', index_in_list=0)
+    clip = ClipAxes('clip')
 
     @classmethod
     def _get_sample_projection(cls):
@@ -1271,17 +1465,21 @@ class MapPlotter(Base2D):
         self._ax = value
 
 
-class FieldPlotter(Simple2DBase, MapPlotter, BasePlotter):
+class FieldPlotter(psyps.Simple2DBase, MapPlotter, psyps.BasePlotter):
     """Plotter for 2D scalar fields on a map
     """
 
     _rcparams_string = ['plotter.fieldplotter']
 
-    levels = ContourLevels('levels', cbounds='bounds')
+    levels = psyps.ContourLevels('levels', cbounds='bounds')
+    try:
+        interp_bounds = psyps.InterpolateBounds('interp_bounds')
+    except AttributeError:
+        pass
     plot = MapPlot2D('plot')
 
 
-class VectorPlotter(MapPlotter, BaseVectorPlotter, BasePlotter):
+class VectorPlotter(MapPlotter, psyps.BaseVectorPlotter, psyps.BasePlotter):
     """Plotter for visualizing 2-dimensional vector data on a map
 
     See Also
@@ -1297,7 +1495,7 @@ class VectorPlotter(MapPlotter, BaseVectorPlotter, BasePlotter):
     color = MapVectorColor('color')
 
 
-class CombinedPlotter(CombinedBase, FieldPlotter, VectorPlotter):
+class CombinedPlotter(psyps.CombinedBase, FieldPlotter, VectorPlotter):
     """Combined 2D plotter and vector plotter on a map
 
     See Also
